@@ -22,6 +22,7 @@ import json
 import uuid
 from contextlib import redirect_stdout
 from io import StringIO
+from typing import TYPE_CHECKING
 
 import httpx
 import pytest
@@ -74,6 +75,9 @@ from airflowctl.api.datamodels.generated import (
     VersionInfo,
 )
 
+if TYPE_CHECKING:
+    from pydantic import NonNegativeInt
+
 
 def make_api_client(
     transport: httpx.MockTransport | None = None,
@@ -165,7 +169,7 @@ class TestAssetsOperations:
 
 
 class TestBackfillOperations:
-    backfill_id: int = 1
+    backfill_id: NonNegativeInt = 1
 
     def test_create(self):
         backfill_body = BackfillPostBody(
@@ -657,10 +661,12 @@ class TestVariablesOperations:
     key = "key"
     value = "val"
     description = "description"
-    variable = VariableBody(
-        key=key,
-        value=value,
-        description=description,
+    variable = VariableBody.model_validate(
+        {
+            "key": key,
+            "value": value,
+            "description": description,
+        }
     )
     variable_response = VariableResponse(
         key=key,
